SCALER = "scaler"
UNIT = "unit"
DAY_ID = "day id"
DAY_SCHEDULE = "day schedule"
START_TIME = "start time"
END_TIME = "end time"
STOP_TIME = "stop time"
SCRIPT_LOGICAL_NAME = "script logical name"
SCRIPT_SELECTOR = "script selector"
SEASON_PROFILE_NAME = "season profile name"
SEASON_START = "season start"
WEEK_NAME = "week name"
WEEK_PROFILE_NAME = "week profile name"
MONDAY = "monday"
TUESDAY = "tuesday"
WEDNESDAY = "wednesday"
THURSDAY = "thursday"
FRIDAY = "friday"
SATURDAY = "saturday"
SUNDAY = "sunday"
ITE_OPTICAL_PORT_NAME = "optical port name"
ITE_RS485_NAME = "rs485 name"
PRESET_TIME = "preset time"
VALIDITY_INTERVAL_START = "validity interval start"
VALIDITY_INTERVAL_END = "validity interval end"
PRECEDENCE = "precedence"
DELAY = "delay"
RELIABILITY = "reliability"
PEAK_THROUGHPUT = "peak throughput"
MEAN_THROUGHPUT = "mean throughput"
DEFAULT = "default"
REQUESTED = "requested"
CELL_ID = "cell ID"
SIGNAL_QUALITY = "signal quality"
LOCATION_ID = "location ID"
BER = "ber"
IMAGE_BLOCK_NUMBER = "image block number"
IMAGE_BLOCK_VALUE = "image block value"
IMAGE_TO_ACTIVATE_SIZE = "image to activate size"
IMAGE_TO_ACTIVATE_IDENTIFICATION = "image to activate identification"
IMAGE_TO_ACTIVATE_SIGNATURE = "image to activate signature"
IMAGE_IDENTIFIER = "image identifier"
IMAGE_SIZE = "image size"
ACTION_OVER_THRESHOLD = "action over threshold"
ACTION_UNDER_THRESHOLD = "action under threshold"
EMERGENCY_PROFILE_ID = "emergency profile ID"
EMERGENCY_ACTIVATION_TIME = "emergency activation time"
EMERGENCY_DURATION = "emergency duration"
CLASS_ID = "class id"
LOGICAL_NAME = "logical name"
ATTRIBUTE_INDEX = "attribute index"
DATA_INDEX = "data index"
FROM_DATE = "from date"
TO_DATE = "to date"
FROM_ENTRY = "from entry"
TO_ENTRY = "to entry"
FROM_SELECTED_VALUE = "from selected value"
TO_SELECTED_VALUE = "to selected value"
RESTRICTING_OBJECT = "restricting object"
FROM_VALUE = "from value"
TO_VALUE = "to value"
SELECTED_VALUES = "selected values"
FIRST_INDEX = "first index"
LAST_INDEX = "last index"
FIRST_INDEX_A = "first index A"
LAST_INDEX_A = "last index A"
FIRST_INDEX_B = "first index B"
LAST_INDEX_B = "last index B"
INDEX = "index"
ENABLE = "enable"
SWITCH_TIME = "switch time"
VALIDITY_WINDOW = "validity window"
EXEC_WEEKDAYS = "exec weekdays"
EXEC_SPEC_DAYS = "exec spec days"
BEGIN_DATE = "begin date"
END_DATE = "end date"
SERVICE_ID = "service ID"
PARAMETER = "parameter"
SCRIPT_IDENTIFIER = "script identifier"
ACTIONS = "actions"
TIME = "time"
DATE = "date"
SPECIAL_DAY_DATE = "special_day_date"
REQUEST = "request"
RESPONSE = "response"
DELAY_AFTER_RESPONSE = "delay after response"
ATTRIBUTE_ACCESS = "attribute access"
METHOD_ACCESS = "method access"
JOINT_ISO_CTT_ELEMENT = "joint iso ctt element"
COUNTRY_ELEMENT = "country element"
COUNTRY_NAME_ELEMENT = "country name element"
IDENTIFIED_ORGANIZATION_ELEMENT = "identified organization element"
DLMS_UA_ELEMENT = "DLMS ua element"
APPLICATION_CONTEXT_ELEMENT = "application context element"
CONTEXT_ID_ELEMENT = "context ID element"
CLIENT_SAP = "client SAP"
SERVER_SAP = "server SAP"
ATTRIBUTE_ID = "attribute ID"
ACCESS_MODE = "access mode"
ACCESS_SELECTORS = "access selectors"
AUTHENTICATION_MECHANISM_NAME_ELEMENT = "authentication mechanism name element"
MECHANISM_ID_ELEMENT = "mechanism id element"
METHOD_ID = "method ID"
VERSION = "version"
ACCESS_RIGHTS = "access rights"
CONFORMANCE = "conformance"
MAX_RECEIVE_PDU_SIZE = "max receive pdu size"
MAX_SEND_PDU_SIZE = "max send pdu size"
DLMS_VERSION_NUMBER = "DLMS version number"
QUALITY_OF_SERVICE = "quality of service"
CYPHERING_INFO = "cyphering info"
KEY_ID = "key ID"
KEY_WRAPPED = "key wrapped"
KEY_DATA = "key data"
CERTIFICATE_ENTITY = "certificate entity"
CERTIFICATE_TYPE = "certificate type"
SERIAL_NUMBER = "serial number"
ISSUER = "issuer"
SUBJECT = "subject"
SUBJECT_ALT_NAME = "subject alt name"
SYSTEM_TITLE = "system title"
CERTIFICATE_IDENTIFICATION_TYPE = "certificate identification type"
CERTIFICATION_IDENTIFICATION_OPTIONS = "certification identification options"
REQUEST_ACTOR = "request actor"
REQUEST_ACTION_LIST = "request action list"
ACTION_UP = "action up"
ACTION_DOWN = "action down"
ITE_BIT_MAP = "BitMap data"
RESTRICTION = "restriction"
RESTRICTION_TYPE = "restriction type"
RESTRICTION_VALUE = "restriction value"
COLUMN = "column"
TRANSPORT_SERVICE = "transport service"
DESTINATION = "destination"
MESSAGE = "message"
REPETITION_DELAY_MIN = "repetition delay min"
REPETITION_DELAY_EXPONENT = "repetition delay exponent"
REPETITION_DELAY_MAX = "repetition delay max"
CONFIRMATION_START_DATE = "confirmation start date"
CONFIRMATION_INTERVAL = "confirmation interval"
KEK_ID = "kek id"
KEY_PARAMETERS = "key parameters"
KEY_CIPHERED_DATA = "key ciphered data"
KEY_INFO_TYPE = "key info type"
KEY_INFO_OPTIONS = "key info options"
KEY_INFO = "key info"
TRANSACTION_ID = "transaction id"
ORIGINATOR_SYSTEM_TITLE = "originator system title"
RECIPIENT_SYSTEM_TITLE = "recipient system title"
OTHER_INFORMATION = "other information"
PROTECTION_TYPE = "protection type"
PROTECTION_OPTIONS = "protection options"
