DATA = 1
REGISTER = 3
EXT_REGISTER = 4
DEMAND_REGISTER = 5
REGISTER_ACTIVATION = 6
PROFILE_GENERIC = 7
CLOCK = 8
SCRIPT_TABLE = 9
SCHEDULE = 10
SPECIAL_DAYS_TABLE = 11
ASSOCIATION_SN_CLASS = 12
ASSOCIATION_LN_CLASS = 15
IMAGE_TRANSFER = 18
IEC_LOCAL_PORT_SETUP = 19
ACTIVITY_CALENDAR = 20
REGISTER_MONITOR = 21
SINGLE_ACTION_SCHEDULE = 22
IEC_HDLC_SETUP = 23
IEC_TWISTED_PAIR__1__SETUP = 24
M_BUS_SLAVE_PORT_SETUP = 25
UTILITY_TABLES = 26
MODEM_CONFIGURATION = 27
AUTO_ANSWER = 28
AUTO_CONNECT = 29
COSEM_DATA_PROTECTION = 30
PUSH_SETUP = 40
TCP_UDP_SETUP = 41
IPV4_SETUP = 42
PRIME_NB_OFDM_PLC_MAC_ADDRESS_SETUP = 43
PPP_SETUP = 44
GPRS_MODEM_SETUP = 45
SMTP_SETUP = 46
GSM_DIAGNOSTIC = 47
IPV6_SETUP = 48
S_FSK_PHY_MAC_SET_UP = 50
S_FSK_ACTIVE_INITIATOR = 51
S_FSK_MAC_SYNCHRONIZATION_TIMEOUTS = 52
S_FSK_MAC_COUNTERS = 53
IEC_61334_4_32_LLC_SETUP = 55
S_FSK_REPORTING_SYSTEM_LIST = 56
ISO_IEC_8802_2_LLC_TYPE_1_SETUP = 57
ISO_IEC_8802_2_LLC_TYPE_2_SETUP = 58
ISO_IEC_8802_2_LLC_TYPE_3_SETUP = 59
REGISTER_TABLE = 61
COMPACT_DATA = 62
STATUS_MAPPING = 63
SECURITY_SETUP = 64
PARAMETER_MONITOR = 65
SENSOR_MANAGER_INTERFACE_CLASS = 67
ARBITRATOR = 68
DISCONNECT_CONTROL = 70
LIMITER = 71
M_BUS_CLIENT = 72
WIRELESS_MODE_Q_CHANNEL = 73
M_BUS_MASTER_PORT_SETUP = 74
DLMS_COSEM_SERVER_M_BUS_PORT_SETUP = 76
M_BUS_DIAGNOSTIC = 77
_61334_4_32_LLC_SSCS_SETUP = 80
PRIME_NB_OFDM_PLC_PHYSICAL_LAYER_COUNTERS = 81
PRIME_NB_OFDM_PLC_MAC_SETUP = 82
PRIME_NB_OFDM_PLC_MAC_FUNCTIONAL_PARAMETERS = 83
PRIME_NB_OFDM_PLC_MAC_COUNTERS = 84
PRIME_NB_OFDM_PLC_MAC_NETWORK_ADMINISTRATION_DATA = 85
PRIME_NB_OFDM_PLC_APPLICATION_IDENTIFICATION = 86
G3_PLC_MAC_LAYER_COUNTERS = 90
NTP_SETUP = 100
ZIGBEE_SAS_STARTUP = 101
ZIGBEE_SAS_JOIN = 102
ZIGBEE_SAS_APS_FRAGMENTATION = 103
ZIGBEE_NETWORK_CONTROL = 104
ZIGBEE_TUNNEL_SETUP = 105
ACCOUNT = 111
CREDIT_INTERFACE_CLASS = 112
CHARGE = 113
TOKEN_GATEWAY = 115
ARRAY_MANAGER = 123
COMMUNICATION_PORT_PROTECTION = 124
HS_PLC_ISO_IEC_12139_1_CPAS_SETUP = 141
HS_PLC_ISO_IEC_12139_1_IP_SSAS_SETUP = 142
HS_PLC_ISO_IEC_12139_1_HDLC_SSAS_SETUP = 143

CLIENT_SETUP = 32767
