NULL_DATA = "null-data"
ARRAY = "array"
STRUCTURE = "structure"
BOOLEAN = "boolean"
BIT_STRING = "bit-string"
DOUBLE_LONG = "double-long"
DOUBLE_LONG_UNSIGNED = "double-long-unsigned"
OCTET_STRING = "octet-string"
VISIBLE_STRING = "visible-string"
UTF8_STRING = "utf8-string"
BCD = "bcd"
INTEGER = "integer"
LONG = "long"
UNSIGNED = "unsigned"
LONG_UNSIGNED = "long-unsigned"
COMPACT_ARRAY = "compact-array"
LONG64 = "long64"
LONG64_UNSIGNED = "unsigned-long64"
ENUM = "enum"
FLOAT32 = "float32"
FLOAT64 = "float64"
DATE_TIME = "date-time"
DATE = "date"
TIME = "time"
DON_T_CARE = "dont-care"
