CHANNEL_COMMUNICATION = "Канал связи"
SERIAL_PORT_NAMES = "Имена последовательных портов"
CLIENT_NAME = "Имя клиента"

LOGICAL_NAME = "Логическое имя объекта"
VALUE = "Значение"
CALENDAR_NAME_ACTIVE = "Имя активного календаря"
SEASON_PROFILE_ACTIVE = "Таблица активных сезонных профилей"
WEEK_PROFILE_TABLE_ACTIVE = "Таблица активных недельных профилей"
DAY_PROFILE_TABLE_ACTIVE = "Таблица активных суточных профилей"
CALENDAR_NAME_PASSIVE = "Имя пассивного календаря"
SEASON_PROFILE_PASSIVE = "Таблица пассивных сезонных профилей"
ACTIVATE_PASSIVE_CALENDAR_TIME = "Дата активирования пассивного календаря"
WEEK_PROFILE_TABLE_PASSIVE = "Таблица пассивных недельных профилей"
DAY_PROFILE_TABLE_PASSIVE = "Таблица пассивных суточных профилей"
TIME = "Время"
TIME_ZONE = "Часовой пояс"
STATUS = "Статус"
DAYLIGHT_SAVINGS_BEGIN = "Дата перехода на летнее время"
DAYLIGHT_SAVINGS_END = "Дата перехода на зимнее время"
DAYLIGHT_SAVINGS_DEVIATION = "Сдвиг летнего времени"
DAYLIGHT_SAVINGS_ENABLED = "Разрешение перевода на летнее время"
CLOCK_BASE = "Источник времени"
OUTPUT_STATE = "Состояние выключателя"
CONTROL_STATE = "Статус управления"
CONTROL_MODE = "Режим управления"
SCALER_UNIT = "Масштаб и ед. измерения"
CAPTURE_TIME = "Время захвата"
APN = "APN"
PIN_CODE = "ПИН код"
QUALITY_OF_SERVICE = "Качество обслуживания"
OPERATOR = "Оператор"
CS_ATTACHMENT = "Вложения схем переключения"
PS_STATUS = "Статус переключаемых пакетов"
CELL_INFO = "Информация о соединении"
ADJACENT_CELL = "Соседние соединения"
COMM_SPEED = "Скорость обмена"
WINDOWS_SIZE_TRANSMIT = "Размер окна передачи"
WINDOWS_SIZE_RECEIVE = "Размер окна приема"
MAX_INFO_FIELD_LENGTH_TRANSMIT = "Максимальная длина поля данных при передаче"
MAX_INFO_FIELD_LENGTH_RECEIVE = "Максимальная длина поля данных при приеме"
INTER_OCTET_TIME_OUT = "Межсимвольный таймаут"
INACTIVITY_TIME_OUT = "Межкадровый таймаут"
DEVICE_ADDRESS = "Адрес устройства"
IMAGE_BLOCK_SIZE = "Размер блока данных"
IMAGE_TRANSFERRED_BLOCKS_STATUS = "Статус передачи блоков данных"
IMAGE_FIRST_NOT_TRANSFERRED_BLOCK_NUMBER = "Номер первого непереданного блока"
IMAGE_TRANSFER_ENABLED = "Разрешение передачи данных"
IMAGE_TRANSFER_STATUS = "Статус блока данных"
IMAGE_TO_ACTIVATE_INFO = "Информация для активации"
DL_REFERENCE = "Ссылка на данные"
IP_ADDRESS = "IP адрес"
MONITORED_VALUE = "Контролируемая величина"
THRESHOLD_ACTIVE = "Активный порог"
THRESHOLD_NORMAL = "Нормальный порог"
THRESHOLD_EMERGENCY = "Аварийный порог"
MIN_OVER_THRESHOLD_DURATION = "Мин. длительность превышения порога"
MIN_UNDER_THRESHOLD_DURATION = "Мин. длительность снижения ниже порога"
EMERGENCY_PROFILE = "Аварийный профиль"
EMERGENCY_PROFILE_GROUP_ID_LIST = "Список аварийных профилей"
EMERGENCY_PROFILE_ACTIVE = "Активный аварийный профиль"
ACTIONS = "Действия"
BUFFER = "Буфер данных"
CAPTURE_OBJECTS = "Список захватываемых объектов"
CAPTURE_PERIOD = "Период захвата"
SORT_METHOD = "Метод сортировки"
SORT_OBJECT = "Объект сортировки"
ENTRIES_IN_USE = "Занятых записей"
PROFILE_ENTRIES = "Всего записей"
ENTRIES = "Записи"
SCRIPTS = "Сценарий"
EXECUTED_SCRIPT = "Выполненный сценарий"
TYPE = "Тип"
EXECUTION_TIME = "Время исполнения"
TCP_UDP_PORT = "TCP_UDP_порт"
IP_REFERENCE = "IP ссылка"
MMS = "Максимальный размер сегмента"
NB_OF_SIM_CONN = "Кол-во одновременных соединений"
# Настройки безопасности
SECURITY_POLICY = "Политика безопасности"
SECURITY_SUITE = "Комплект безопасности"
CLIENT_SYSTEM_TITLE = "Название клиента"
SERVER_SYSTEM_TITLE = "Название сервера"
CERTIFICATES = "Сертификаты"
# Арбитр
PERMISSION_TABLE = "Таблица разрешений"
WEIGHTINGS_TABLE = "Таблица весов"
MOST_RECENT_REQUESTS_TABLE = "Таблица последних запросов"
LAST_OUTCOME = "Последний результат"
# Регистр контроля
THRESHOLDS = "Пороги"
#
INITIALIZATION_STRING = "Инициализационые строки"
MODEM_PROFILE = "Профиль модема"
OBJECT_LIST = "Список объектов"
ASSOCIATED_PARTNERS_ID = "Идентификатор партнера"
APPLICATION_CONTEXT_NAME = "Имя контекста приложения"
XDLMS_CONTEXT_INFO = "xDLMS информация"
AUTHENTICATION_MECHANISM_NAME = "Имя алгоритма проверки подлинности"
LLS_SECRET = "Секрет (пароль)"
ASSOCIATION_STATUS = "Статус соединения"
SECURITY_SETUP_REFERENCE = "OBIS-код настройки безопасности"
# Push setup
PUSH_OBJECT_LIST = "Список передаваемых объектов"
SEND_DESTINATION_AND_METHOD = "Метод доставки и адресат"
COMMUNICATION_WINDOW = "Окна связи"
RANDOMISATION_START_INTERVAL = "Интервал псевдослучайной задержки выхода"
NUMBER_OF_RETRIES = "Количество повторов"
REPETITION_DELAY = "Задержка повтора"
PORT_REFERENCE = "Настройка порта"
PUSH_CLIENT_SAP = "Номер клиента SAP"
PUSH_PROTECTION_PARAMETERS = "Параметры защиты"
PUSH_OPERATION_METHOD = "Метод работы"
CONFIRMATION_PARAMETERS = "Параметры подтверждения"
LAST_CONFIRMATION_DATE_TIME = "Время последнего подтверждения"
