CHANNEL_COMMUNICATION = "channel_communication"
SERIAL_PORT_NAMES = "serial_port_names"
CLIENT_NAME = "client_name"

LOGICAL_NAME = "logical name"
VALUE = "value"
CALENDAR_NAME_ACTIVE = "calendar name active"
SEASON_PROFILE_ACTIVE = "season profile active"
WEEK_PROFILE_TABLE_ACTIVE = "week profile table active"
DAY_PROFILE_TABLE_ACTIVE = "day profile table active"
CALENDAR_NAME_PASSIVE = "calendar name passive"
SEASON_PROFILE_PASSIVE = "season profile passive"
WEEK_PROFILE_TABLE_PASSIVE = "week_profile_table_passive"
DAY_PROFILE_TABLE_PASSIVE = "day_profile_table_passive"
ACTIVATE_PASSIVE_CALENDAR_TIME = "activate passive calendar time"
TIME = "time"
TIME_ZONE = "time_zone"
STATUS = "status"
DAYLIGHT_SAVINGS_BEGIN = "daylight_savings_begin"
DAYLIGHT_SAVINGS_END = "daylight_savings_end"
DAYLIGHT_SAVINGS_DEVIATION = "daylight_savings_deviation"
DAYLIGHT_SAVINGS_ENABLED = "daylight_savings_enabled"
CLOCK_BASE = "clock_base"
OUTPUT_STATE = "output_state"
CONTROL_STATE = "control_state"
CONTROL_MODE = "control_mode"
SCALER_UNIT = "scaler_unit"
CAPTURE_TIME = "capture_time"
APN = "APN"
PIN_CODE = "PIN_code"
QUALITY_OF_SERVICE = "quality_of_service"
OPERATOR = "operator"
CS_ATTACHMENT = "cs_attachment"
PS_STATUS = "ps_status"
CELL_INFO = "cell_info"
ADJACENT_CELL = "adjacent_cell"
COMM_SPEED = "comm_speed"
WINDOWS_SIZE_TRANSMIT = "windows_size_transmit"
WINDOWS_SIZE_RECEIVE = "windows_size_receive"
MAX_INFO_FIELD_LENGTH_TRANSMIT = "max_info_field_length_transmit"
MAX_INFO_FIELD_LENGTH_RECEIVE = "max_info_field_length_receive"
INTER_OCTET_TIME_OUT = "inter_octet_time_out"
INACTIVITY_TIME_OUT = "inactivity_time_out"
DEVICE_ADDRESS = "device_address"
IMAGE_BLOCK_SIZE = "image_block_size"
IMAGE_TRANSFERRED_BLOCKS_STATUS = "image_transferred_blocks_status"
IMAGE_FIRST_NOT_TRANSFERRED_BLOCK_NUMBER = "image_first_not_transferred_block_number"
IMAGE_TRANSFER_ENABLED = "image_transfer_enabled"
IMAGE_TRANSFER_STATUS = "image_transfer_status"
IMAGE_TO_ACTIVATE_INFO = "image_to_activate_info"
DL_REFERENCE = "DL_reference"
IP_ADDRESS = "IP_address"
MONITORED_VALUE = "monitored_value"
THRESHOLD_ACTIVE = "threshold_active"
THRESHOLD_NORMAL = "threshold_normal"
THRESHOLD_EMERGENCY = "threshold_emergency"
MIN_OVER_THRESHOLD_DURATION = "min_over_threshold_duration"
MIN_UNDER_THRESHOLD_DURATION = "min_under_threshold_duration"
EMERGENCY_PROFILE = "emergency_profile"
EMERGENCY_PROFILE_GROUP_ID_LIST = "emergency_profile_group_id_list"
EMERGENCY_PROFILE_ACTIVE = "emergency_profile_active"
ACTIONS = "actions"
BUFFER = "buffer"
CAPTURE_OBJECTS = "capture_objects"
CAPTURE_PERIOD = "capture_period"
SORT_METHOD = "sort_method"
SORT_OBJECT = "sort_object"
ENTRIES_IN_USE = "entries_in_use"
PROFILE_ENTRIES = "profile_entries"
ENTRIES = "entries"
SCRIPTS = "scripts"
EXECUTED_SCRIPT = "executed_script"
TYPE = "type"
EXECUTION_TIME = "execution_time"
TCP_UDP_PORT = "TCP_UDP_port"
IP_REFERENCE = "IP_reference"
MMS = "MMS"
NB_OF_SIM_CONN = "nb_of_sim_conn"
# Security setup
SECURITY_POLICY = "security_policy"
SECURITY_SUITE = "security_suite"
CLIENT_SYSTEM_TITLE = "client_system_title"
SERVER_SYSTEM_TITLE = "server_system_title"
CERTIFICATES = "certificates"
# Arbitrator
PERMISSION_TABLE = "permission_table"
WEIGHTINGS_TABLE = "weightings_table"
MOST_RECENT_REQUESTS_TABLE = "most_recent_requests_table"
LAST_OUTCOME = "last_outcome"
# Register monitor
THRESHOLDS = "thresholds"
#
INITIALIZATION_STRING = "initialization_string"
MODEM_PROFILE = "modem_profile"
OBJECT_LIST = "object_list"
ASSOCIATED_PARTNERS_ID = "associated_partners_id"
APPLICATION_CONTEXT_NAME = "application_context_name"
XDLMS_CONTEXT_INFO = "xDLMS_context_info"
AUTHENTICATION_MECHANISM_NAME = "authentication_mechanism_name"
LLS_SECRET = "LLS_secret"
ASSOCIATION_STATUS = "association_status"
SECURITY_SETUP_REFERENCE = "security_setup_reference"
# Push setup
PUSH_OBJECT_LIST = "push_object_list"
SEND_DESTINATION_AND_METHOD = "send_destination_and_method"
COMMUNICATION_WINDOW = "communication_window"
RANDOMISATION_START_INTERVAL = "randomisation_start_interval"
NUMBER_OF_RETRIES = "number_of_retries"
REPETITION_DELAY = "repetition_delay"
PORT_REFERENCE = "port_reference"
PUSH_CLIENT_SAP = "push_client_SAP"
PUSH_PROTECTION_PARAMETERS = "push_protection_parameters"
PUSH_OPERATION_METHOD = "push_operation_method"
CONFIRMATION_PARAMETERS = "confirmation_parameters"
LAST_CONFIRMATION_DATE_TIME = "last_confirmation_date_time"
