FAILED_TO_RECEIVE = "Failed to receive reply from the device in given time"
OPEN_PORT = "Open port"
INVALID_PDU = "Invalid PDU"
UPDATE_AVAILABLE = "update available"
UNKNOWN_APPVERSION_TYPE = "Unknown Appversion type"
INITIALIZE_CONNECTION = "Initialize connection"
