EXECUTE = "execute"
REPLY_TO_HLS_AUTHENTICATION = "reply_to_HLS_authentication"
CHANGE_HLS_SECRET = "change_HLS_secret"
ADD_OBJECT = "add_object"
REMOVE_OBJECT = "remove_object"
ACTIVATE_PASSIVE_CALENDAR = "activate_passive_calendar"
ADJUST_TO_QUARTER = "adjust_to_quarter"
ADJUST_TO_MEASURING_PERIOD = "adjust_to_measuring_period"
ADJUST_TO_MINUTE = "adjust_to_minute"
ADJUST_TO_PRESET_TIME = "adjust_to_preset_time"
PRESET_ADJUSTING_TIME = "preset_adjusting_time"
SHIFT_TIME = "shift_time"
REMOTE_DISCONNECT = "remote_disconnect"
REMOTE_RECONNECT = "remote_reconnect"
RESET = "reset"
IMAGE_TRANSFER_INITIATE = "image_transfer_initiate"
IMAGE_BLOCK_TRANSFER = "image_block_transfer"
IMAGE_VERIFY = "image_verify"
IMAGE_ACTIVATE = "image_activate"
CAPTURE = "capture"
ENABLE_DISABLE = "enable_disable"
INSERT = "insert"
DELETE = "delete"
SECURITY_ACTIVATE = "security_activate"
GLOBAL_KEY_TRANSFER = "global_key_transfer"
KEY_TRANSFER = "key_transfer"
KEY_AGREEMENT = "key agreement"
GENERATE_KEY_PAIR = "generate key pair"
GENERATE_CERTIFICATE_REQUEST = "generate certificate request"
IMPORT_CERTIFICATE = "import certificate"
EXPORT_CERTIFICATE = "export certificate"
REMOVE_CERTIFICATE = "remove certificate"
REQUEST_ACTION = "request_action"
PUSH = "push"
