NULL_DATA = "Отсутствие данных"
ARRAY = "Массив"
STRUCTURE = "Структура"
BOOLEAN = "Логические данные"
BIT_STRING = "Последовательность битов"
DOUBLE_LONG = "32-разр целое со знаком"
DOUBLE_LONG_UNSIGNED = "32-разр целое без знака"
OCTET_STRING = "Последовательность байтов"
VISIBLE_STRING = "Последовательность ASCII символов"
UTF8_STRING = "Последовательность символов UTF-8"
BCD = "Двоично-десятичная кодировка байта"
INTEGER = "8-разр целое число со знаком"
LONG = "16-разр целое число со знаком"
UNSIGNED = "8-разр целое число без знака"
LONG_UNSIGNED = "16-разр целое число без знака"
COMPACT_ARRAY = "Массив упакованных данных"
LONG64 = "64-разрядное целое со знаком"
LONG64_UNSIGNED = "64-разрядное целое без знака"
ENUM = "Перечисление"
FLOAT32 = "4-байтовая строка - число с плавающей запятой"
FLOAT64 = "8-байтовая строка - число с плавающей запятой"
DATE_TIME = "Дата-время"
DATE = "Дата"
TIME = "Время"
DON_T_CARE = "dont-care"
